@@ -36,6 +36,7 @@ import (
 	dockerv10 "github.com/openshift/api/image/docker10"
 	imagev1 "github.com/openshift/api/image/v1"
 	"github.com/openshift/api/network"
+	networkv1 "github.com/openshift/api/network/v1"
 	"github.com/openshift/api/oauth"
 	"github.com/openshift/api/project"
 	projectv1 "github.com/openshift/api/project/v1"
@@ -132,6 +133,7 @@ func describerMap(clientConfig *rest.Config, kclient kubernetes.Interface, host
 		image.Kind("Image"):                          &ImageDescriber{imageClient},
 		image.Kind("ImageStream"):                    &ImageStreamDescriber{imageClient},
 		image.Kind("ImageStreamTag"):                 &ImageStreamTagDescriber{imageClient},
+		image.Kind("ImageTag"):                       &ImageTagDescriber{imageClient},
 		image.Kind("ImageStreamImage"):               &ImageStreamImageDescriber{imageClient},
 		route.Kind("Route"):                          &RouteDescriber{routeClient, kclient},
 		project.Kind("Project"):                      &ProjectDescriber{projectClient, kclient},
@@ -714,7 +716,7 @@ func DescribeImage(image *imagev1.Image, imageName string) (string, error) {
 				}
 			}
 		}
-		formatString(out, "Image Created", fmt.Sprintf("%s ago", formatRelativeTime(dockerImage.Created.Time)))
+		formatString(out, "Image Created", fmt.Sprintf("%s ago", FormatRelativeTime(dockerImage.Created.Time)))
 		formatString(out, "Author", dockerImage.Author)
 		formatString(out, "Arch", dockerImage.Architecture)
 
@@ -799,6 +801,30 @@ func (d *ImageStreamTagDescriber) Describe(namespace, name string, settings desc
 	return DescribeImage(&imageStreamTag.Image, imageStreamTag.Image.Name)
 }
 
+// ImageTagDescriber generates information about a ImageTag (Image).
+type ImageTagDescriber struct {
+	c imageclient.ImageV1Interface
+}
+
+// Describe returns the description of an imageStreamTag
+func (d *ImageTagDescriber) Describe(namespace, name string, settings describe.DescriberSettings) (string, error) {
+	c := d.c.ImageTags(namespace)
+	repo, tag, err := imageutil.ParseImageStreamTagName(name)
+	if err != nil {
+		return "", err
+	}
+	if len(tag) == 0 {
+		// TODO use repo's preferred default, when that's coded
+		tag = imagev1.DefaultImageTag
+	}
+	imageTag, err := c.Get(repo+":"+tag, metav1.GetOptions{})
+	if err != nil {
+		return "", err
+	}
+
+	return DescribeImage(imageTag.Image, imageTag.Image.Name)
+}
+
 // ImageStreamImageDescriber generates information about a ImageStreamImage (Image).
 type ImageStreamImageDescriber struct {
 	c imageclient.ImageV1Interface
@@ -888,9 +914,9 @@ func (d *RouteDescriber) Describe(namespace, name string, settings describe.Desc
 				}
 				switch status, condition := routedisplayhelpers.IngressConditionStatus(&ingress, routev1.RouteAdmitted); status {
 				case corev1.ConditionTrue:
-					fmt.Fprintf(out, "\t  exposed on router %s%s %s ago\n", ingress.RouterName, hostName, strings.ToLower(formatRelativeTime(condition.LastTransitionTime.Time)))
+					fmt.Fprintf(out, "\t  exposed on router %s%s %s ago\n", ingress.RouterName, hostName, strings.ToLower(FormatRelativeTime(condition.LastTransitionTime.Time)))
 				case corev1.ConditionFalse:
-					fmt.Fprintf(out, "\t  rejected by router %s: %s%s (%s ago)\n", ingress.RouterName, hostName, condition.Reason, strings.ToLower(formatRelativeTime(condition.LastTransitionTime.Time)))
+					fmt.Fprintf(out, "\t  rejected by router %s: %s%s (%s ago)\n", ingress.RouterName, hostName, condition.Reason, strings.ToLower(FormatRelativeTime(condition.LastTransitionTime.Time)))
 					if len(condition.Message) > 0 {
 						fmt.Fprintf(out, "\t    %s\n", condition.Message)
 					}
@@ -910,9 +936,9 @@ func (d *RouteDescriber) Describe(namespace, name string, settings describe.Desc
 			}
 			switch status, condition := routedisplayhelpers.IngressConditionStatus(&ingress, routev1.RouteAdmitted); status {
 			case corev1.ConditionTrue:
-				fmt.Fprintf(out, "\t%s exposed on router %s %s%s ago\n", ingress.Host, ingress.RouterName, hostName, strings.ToLower(formatRelativeTime(condition.LastTransitionTime.Time)))
+				fmt.Fprintf(out, "\t%s exposed on router %s %s%s ago\n", ingress.Host, ingress.RouterName, hostName, strings.ToLower(FormatRelativeTime(condition.LastTransitionTime.Time)))
 			case corev1.ConditionFalse:
-				fmt.Fprintf(out, "\trejected by router %s: %s%s (%s ago)\n", ingress.RouterName, hostName, condition.Reason, strings.ToLower(formatRelativeTime(condition.LastTransitionTime.Time)))
+				fmt.Fprintf(out, "\trejected by router %s: %s%s (%s ago)\n", ingress.RouterName, hostName, condition.Reason, strings.ToLower(FormatRelativeTime(condition.LastTransitionTime.Time)))
 				if len(condition.Message) > 0 {
 					fmt.Fprintf(out, "\t  %s\n", condition.Message)
 				}
@@ -1776,12 +1802,34 @@ func (d *HostSubnetDescriber) Describe(namespace, name string, settings describe
 		formatString(out, "Node", hs.Host)
 		formatString(out, "Node IP", hs.HostIP)
 		formatString(out, "Pod Subnet", hs.Subnet)
-		formatString(out, "Egress CIDRs", strings.Join(hs.EgressCIDRs, ", "))
-		formatString(out, "Egress IPs", strings.Join(hs.EgressIPs, ", "))
+		formatString(out, "Egress CIDRs", hostCIDRsJoin(hs.EgressCIDRs, ", "))
+		formatString(out, "Egress IPs", hostIPsJoin(hs.EgressIPs, ", "))
 		return nil
 	})
 }
 
+func hostCIDRsJoin(cidrs []networkv1.HostSubnetEgressCIDR, sep string) string {
+	var b strings.Builder
+	for i, c := range cidrs {
+		b.WriteString(string(c))
+		if i < len(cidrs)-1 {
+			b.WriteString(sep)
+		}
+	}
+	return b.String()
+}
+
+func hostIPsJoin(ips []networkv1.HostSubnetEgressIP, sep string) string {
+	var b strings.Builder
+	for i, c := range ips {
+		b.WriteString(string(c))
+		if i < len(ips)-1 {
+			b.WriteString(sep)
+		}
+	}
+	return b.String()
+}
+
 type NetNamespaceDescriber struct {
 	c onetworktypedclient.NetworkV1Interface
 }
@@ -1796,11 +1844,22 @@ func (d *NetNamespaceDescriber) Describe(namespace, name string, settings descri
 		formatMeta(out, netns.ObjectMeta)
 		formatString(out, "Name", netns.NetName)
 		formatString(out, "ID", netns.NetID)
-		formatString(out, "Egress IPs", strings.Join(netns.EgressIPs, ", "))
+		formatString(out, "Egress IPs", netIPsJoin(netns.EgressIPs, ", "))
 		return nil
 	})
 }
 
+func netIPsJoin(ips []networkv1.NetNamespaceEgressIP, sep string) string {
+	var b strings.Builder
+	for i, c := range ips {
+		b.WriteString(string(c))
+		if i < len(ips)-1 {
+			b.WriteString(sep)
+		}
+	}
+	return b.String()
+}
+
 type EgressNetworkPolicyDescriber struct {
 	c onetworktypedclient.NetworkV1Interface
 }
